"""
LeMMA - the Lazy man's Editor for MMA
Note that this is NOT really an MMA editor, but rather a simple front-end to MMA

This is the module that deals with (mostly) configuration stuff

Software is provided AS IS. Feel free to use AT YOUR OWN RISK!

Version 0.7.1. Copyright (c) Gek S. Low. 26 Mar 2009
"""

# Settings
from Tkinter import *
import os
import re
import tkFileDialog
import tkSimpleDialog
import tkMessageBox
import subprocess
import cPickle
import commands
import platform

from constants import *

import logging

# Global variables
isLinux = None
mmaPath = ""
midiPlayer = ""
pythonPath = ""
customGroovePath = ""

groovesFile = ""
settingsFile = ""

libDir = ""

groovedict = {}
groovelib_lookup = {} # given a groove name and libpath, look up the groove library
currentgroovelib = ""

outputWindow = Text()

def detectPlatform():
	global isLinux
	# Assume Linux platform is default if not Windows
	# The subprocess calls under Windows are handled a bit differently
	if platform.system() not in ("Windows", "Microsoft"):
		isLinux = True
	else:
		isLinux = False
	logging.debug("[detectPlatform] isLinux = " + str(isLinux) + ", platform.system() returns '" + platform.system() + "'")


# AutoScrollbar example taken from http://effbot.org/zone/tkinter-autoscrollbar.htm
class AutoScrollbar(Scrollbar):
        # a scrollbar that hides itself if it's not needed.  only
        # works if you use the grid geometry manager.
        def set(self, lo, hi):
                if float(lo) <= 0.0 and float(hi) >= 1.0:
                        # grid_remove is currently missing from Tkinter!
                        self.tk.call("grid", "remove", self)
                else:
                        self.grid()
                Scrollbar.set(self, lo, hi)

        def pack(self, **kw):
                raise TclError, "cannot use pack with this widget"

        def place(self, **kw):
                raise TclError, "cannot use place with this widget"

def printOutput(text, append=False):
	if append == False:
		outputWindow.delete("1.0", END)
		outputWindow.insert("1.0", text + "\n")
	else:
		outputWindow.insert(END, text + "\n")
	outputWindow.update()

def autoDetectPaths():
	global pythonPath
	global mmaPath

	# Python path
	pythonPath = sys.executable
	if pythonPath != "":
		printOutput("Found python at " + pythonPath, True)
		logging.debug("[AutoDetectPaths] Found python at " + pythonPath)
	else:
		printOutput("Unable to find python!", True)
		logging.debug("[AutoDetectPaths] Found python at " + pythonPath)

	# MMA path
	if os.path.exists('/usr/local/bin/mma'):
		mmaPath = '/usr/local/bin/mma'
		printOutput("Found MMA at " + mmaPath, True)
		logging.debug("[AutoDetectPaths] Found MMA at " + mmaPath)
	elif os.path.exists('/usr/bin/mma'):
		mmaPath = '/usr/bin/mma'
		printOutput("Found MMA at " + mmaPath, True)
		logging.debug("[AutoDetectPaths] Found MMA at " + mmaPath)
	elif os.path.exists('C:\mma\mma.py'):
		mmaPath = 'C:\mma\mma.py'
		printOutput("Found MMA at " + mmaPath, True)
		logging.debug("[AutoDetectPaths] Found MMA at " + mmaPath)
	elif os.path.exists(os.path.normpath(os.getcwd() + '/mma.py')):
		mmaPath = os.path.normpath(os.getcwd() + '/mma.py')
		printOutput("Found MMA at " + mmaPath, True)
		logging.debug("[AutoDetectPaths] Found MMA at " + mmaPath)
	else:
		printOutput("Unable to auto-detect MMA path. Please set it manually in 'Settings'", True)
		logging.debug("[AutoDetectPaths] Unable to auto-detect MMA path")

	# MMA grooves path
	determineLibDir()


def determineLibDir():
	# try different locations for the libraries
	# try the lib directory from mma path first
	global libDir
	global mmaPath
	mmaDir = os.path.dirname(mmaPath)

	if os.path.exists("/usr/local/share/mma/lib"):
		libDir = "/usr/local/share/mma/lib"
		printOutput("Grooves directory is " + libDir, True)
		logging.debug("[determineLibDir] Found " + libDir + " when MMA path is " + mmaPath)
	elif os.path.exists("/usr/share/mma/lib"):
		libDir = "/usr/share/mma/lib"
		printOutput("Grooves directory is " + libDir, True)
		logging.debug("[determineLibDir] Found " + libDir + " when MMA path is " + mmaPath)
	elif os.path.exists(os.path.normpath(mmaDir + "/lib")):
		libDir = os.path.normpath(mmaDir + "/lib")
		printOutput("Grooves directory is " + libDir, True)
		logging.debug("[determineLibDir] Found " + libDir + " when MMA path is " + mmaPath)
	else:
		#libDir = "";
		printOutput("Couldn't find the Grooves directory. Perhaps your MMA path is incorrect?", True)
		logging.debug("[determineLibDir] Unable to determine when MMA path is " + mmaPath)
		return

def readGrooves():
	global groovedict
	global groovelib_lookup
	global groovesFile
	try:
		f = open(groovesFile, "r")
		p = cPickle.Unpickler(f)
		groovedict = p.load()
		groovelib_lookup = p.load()
		f.close()
	except IOError:
		printOutput("Error opening " + groovesFile + ". Please check your settings, then refresh the grooves.", True)

def setConfigPath(configPath):
	global settingsFile
	global groovesFile

	settingsFile = os.path.normpath(configPath + "/settings.dat")
	groovesFile = os.path.normpath(configPath + "/grooves.dat")


def readSettings():
	global mmaPath
	global midiPlayer
	global pythonPath
	global customGroovePath
	global libDir
	global settingsFile
	
	# load settings (if file exists)
	try:
		f = open(settingsFile, "r")
		p = cPickle.Unpickler(f)
		pythonPath = p.load()
		mmaPath = p.load()
		midiPlayer = p.load()
		customGroovePath = p.load()
		libDir = p.load()
		f.close()
	except IOError:
		printOutput("Error opening " + settingsFile + ". Will try to auto-detect. Please check settings later.", True)
		logging.debug("[settings] Cannot find " + settingsFile)
		autoDetectPaths()

def playMMA():
	global mmaPath
	global midiPlayer

	# validate the paths first
	if not os.path.exists(mmaPath):
		tkMessageBox.showerror("Error", "MMA Path: '" + mmaPath + "' does not exist. Please check your settings.")
		return
	realmidipath = re.compile(r'\s-[^\s]*').sub("", midiPlayer)	# strip any command-line options
	if not os.path.exists(realmidipath):
		tkMessageBox.showerror("Error", "Midi player: '" + midiPlayer + "' does not exist. Please check your settings.")
		#return		# non-fatal error in case of bugs
	if os.path.isdir(pythonPath):
		tkMessageBox.showerror("Error", "Python path: '" + pythonPath + "' is a directory and not a filename. Please check your settings.")
		return
	if os.path.isdir(mmaPath):
		tkMessageBox.showerror("Error", "MMA path: '" + mmaPath + "' is a directory and not a filename. Please check your settings.")
		return


	# run MMA - must run from mma.py path
	currentdir = os.getcwd()
	mmaDir = os.path.dirname(mmaPath)
	os.chdir(mmaDir)
	
	cmd = os.path.normcase(pythonPath + " " + mmaPath + " \"" + currentdir + "/_temp_.mma\"")
	logging.debug("[playMMA] Calling process '" + cmd + "'")
	pipe = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE).stdout
	output = pipe.read()
	status = pipe.close()

	os.chdir(currentdir)
	
	#print output
	printOutput(output)
	
	# play Midi
	if isLinux == False:
		# On windows, need shell=False to avoid a shell window
		useShell = False
	else:
		# On Linux, need shell=True for this to work, but note that there is no way to control/stop timidity.
		useShell = True

	# On windows, can't use os.popen as that will wait for process to finish, then I'll have to close the midi player all the time
	midiopts = midiPlayer.replace(realmidipath, "")	# get any command-line options in midiPlayer
	cmd = os.path.normcase("\"" + realmidipath + "\" " + midiopts + " \"" + currentdir + "/_temp_.mid\"")
	logging.debug("[playMMA] Calling process '" + cmd + "'")
	pipe = subprocess.Popen(cmd, shell=useShell, stdout=None).stdout


class ViewFileDialog(tkSimpleDialog.Dialog):
	contents = ""

	def __init__(self, parent, title=None, filename=""):
		try:
			f = open(filename, "r")
			self.contents = f.read()
			f.close()
		except:
			printOutput("Can't open "+filename)
			return
		tkSimpleDialog.Dialog.__init__(self, parent, title)

	def body(self, master):
		self.viewText = Text(master, width=50, height=20, bg="white", wrap=WORD, font=TEXT_FONT)
		self.viewText.insert("1.0", self.contents)
		self.viewText.configure(state=DISABLED)
		self.viewText.grid(row=0, column=0, sticky=N+S+E+W)

		self.viewScrollY = AutoScrollbar(master, orient=VERTICAL, command=self.viewText.yview)
		self.viewScrollY.grid(row=0, column=1, stick=NS)
		self.viewText["yscrollcommand"] = self.viewScrollY.set

		self.viewScrollX = AutoScrollbar(master, orient=HORIZONTAL, command=self.viewText.xview)
		self.viewScrollX.grid(row=1, column=0, stick=EW)
		self.viewText["xscrollcommand"] = self.viewScrollX.set
	
	def apply(self):
		return


class CodeDialog(tkSimpleDialog.Dialog):
	code = ""
	def __init__(self, parent, code=""):
		self.code = code
		tkSimpleDialog.Dialog.__init__(self, parent, title="Insert MMA commands")

	def body(self, master):
		self.codeText = Text(master, width=50, height=20, bg="white", wrap=NONE, font=TEXT_FONT)
		self.codeText.insert("1.0", self.code)
		self.codeText.grid(row=0, column=0, sticky=N+S+E+W)

		self.codeScrollY = AutoScrollbar(master, orient=VERTICAL, command=self.codeText.yview)
		self.codeScrollY.grid(row=0, column=1, stick=NS)
		self.codeText["yscrollcommand"] = self.codeScrollY.set

		self.codeScrollX = AutoScrollbar(master, orient=HORIZONTAL, command=self.codeText.xview)
		self.codeScrollX.grid(row=1, column=0, stick=EW)
		self.codeText["xscrollcommand"] = self.codeScrollX.set
	
		return self.codeText
	
	def apply(self):
		self.code = self.codeText.get("1.0", END)
		return

	def buttonbox(self):
		tkSimpleDialog.Dialog.buttonbox(self)
		# Need to remove the enter key binding as it conflicts with the text widget
		self.unbind("<Return>")
		return

class SettingsDialog(tkSimpleDialog.Dialog):
	def __init__(self, parent):
		tkSimpleDialog.Dialog.__init__(self, parent, title="Settings")

	def body(self, master):
		global mmaPath
		global midiPlayer
		global libDir
		global pythonPath
		global customGroovePath
		
		Label(master, text="Python path ", anchor=W).grid(row=0, column=0, sticky=W)
		self.pythonPathEntry = Entry(master, width=20, font=TEXT_FONT)
		self.pythonPathEntry.insert(1, pythonPath)
		self.pythonPathEntry.grid(row=0, column=1, sticky=E+W)
		self.pythonPathBrowseBtn = Button(master, text="Browse", command=self.pythonPathBrowse)
		self.pythonPathBrowseBtn.grid(row=0, column=2, sticky=NW)
		
		Label(master, text="MMA path ", anchor=W).grid(row=1, column=0, sticky=W)
		self.mmaPathEntry = Entry(master, width=20, font=TEXT_FONT)
		self.mmaPathEntry.insert(1, mmaPath)
		self.mmaPathEntry.grid(row=1, column=1, sticky=E+W)
		self.mmaPathBrowseBtn = Button(master, text="Browse", command=self.mmaPathBrowse)
		self.mmaPathBrowseBtn.grid(row=1, column=2, sticky=NW)

		Label(master, text="MMA grooves path ", anchor=W).grid(row=2, column=0, sticky=W)
		self.mmaLibDirEntry = Entry(master, width=20, font=TEXT_FONT)
		self.mmaLibDirEntry.insert(1, libDir)
		self.mmaLibDirEntry.grid(row=2, column=1, sticky=E+W)
		self.mmaLibDirBrowseBtn = Button(master, text="Browse", command=self.mmaLibDirBrowse)
		self.mmaLibDirBrowseBtn.grid(row=2, column=2, sticky=NW)

		Label(master, text="Midi player ", anchor=W).grid(row=3, column=0, sticky=W)
		self.midiPlayerEntry = Entry(master, width=20, font=TEXT_FONT)
		self.midiPlayerEntry.insert(1, midiPlayer)
		self.midiPlayerEntry.grid(row=3, column=1, sticky=E+W)
		self.midiPlayerBrowseBtn = Button(master, text="Browse", command=self.midiPlayerBrowse)
		self.midiPlayerBrowseBtn.grid(row=3, column=2, sticky=NW)

		Label(master, text="(Optional) Custom \ngroove folder ", anchor=W, justify=LEFT).grid(row=4, column=0, sticky=W)
		self.customGroovePathEntry = Entry(master, width=20, font=TEXT_FONT)
		self.customGroovePathEntry.insert(1, customGroovePath)
		self.customGroovePathEntry.grid(row=4, column=1, sticky=E+W)
		self.customGroovePathBrowseBtn = Button(master, text="Browse", command=self.customGrooveBrowse)
		self.customGroovePathBrowseBtn.grid(row=4, column=2, sticky=NW)

		self.autoDetectBtn = Button(master, text="Auto-detect paths", command=self.autoDetectPaths)
		self.autoDetectBtn.grid(row=5, column=0, columnspan=3)

		self.mmaPathRefreshBtn = Button(master, text="Refresh grooves library", command=self.refreshGrooves)
		self.mmaPathRefreshBtn.grid(row=6, column=0, columnspan=3)

		return self.mmaPathEntry	# initial focus
	
	def autoDetectPaths(self):
		global pythonPath
		global mmaPath
		global libDir

		if tkMessageBox.askyesno("Warning", "Your current settings will be overwritten. Continue?"):
			# read variables and strip trailing spaces	
			pythonPath = self.pythonPathEntry.get().strip(' ')
			mmaPath = self.mmaPathEntry.get().strip(' ')
			libDir = self.mmaLibDirEntry.get().strip(' ')

			autoDetectPaths()
			self.pythonPathEntry.delete(0, END)
			self.pythonPathEntry.insert(1, pythonPath)

			self.mmaPathEntry.delete(0, END)
			self.mmaPathEntry.insert(1, mmaPath)

			self.mmaLibDirEntry.delete(0, END)
			self.mmaLibDirEntry.insert(1, libDir)
	

	def mmaPathBrowse(self):
		currentFile = tkFileDialog.askopenfilename(parent=self)
		if currentFile != "" and currentFile != ():
			currentFile = os.path.normpath(currentFile)
			self.mmaPathEntry.delete(0, END)
			self.mmaPathEntry.insert(1, currentFile)

	def mmaLibDirBrowse(self):
		currentFile = tkFileDialog.askopenfilename(parent=self)
		if currentFile != "" and currentFile != ():
			currentFile = os.path.dirname(os.path.normpath(currentFile)) # dir name
			self.mmaLibDirEntry.delete(0, END)
			self.mmaLibDirEntry.insert(1, currentFile)

	def pythonPathBrowse(self):
		currentFile = tkFileDialog.askopenfilename(parent=self)
		if currentFile != "" and currentFile != ():
			currentFile = os.path.normpath(currentFile)
			self.pythonPathEntry.delete(0, END)
			self.pythonPathEntry.insert(1, currentFile)
	
	def midiPlayerBrowse(self):
		currentFile = tkFileDialog.askopenfilename(parent=self)
		if currentFile != "" and currentFile != ():
			currentFile = os.path.normpath(currentFile)
			self.midiPlayerEntry.delete(0, END)
			self.midiPlayerEntry.insert(1, currentFile)

	def customGrooveBrowse(self):
		currentFile = tkFileDialog.askopenfilename(parent=self)
		if currentFile != "" and currentFile != ():
			currentFile = os.path.dirname(os.path.normpath(currentFile)) # dir name
			self.customGroovePathEntry.delete(0, END)
			self.customGroovePathEntry.insert(1, currentFile)
	
	def apply(self):
		global mmaPath
		global midiPlayer
		global pythonPath
		global customGroovePath
		global libDir
		global settingsFile
		
		# read variables and strip trailing spaces	
		pythonPath = self.pythonPathEntry.get().strip(' ')
		mmaPath = self.mmaPathEntry.get().strip(' ')
		libDir = self.mmaLibDirEntry.get().strip(' ')
		midiPlayer = self.midiPlayerEntry.get().strip(' ')
		customGroovePath = self.customGroovePathEntry.get().strip(' ')

		logging.debug("[Settings] Settings are: (" + pythonPath + "," + mmaPath + "," + midiPlayer + "," + customGroovePath + ")")
		# validate the existence of these paths
		if not os.path.exists(pythonPath):
			tkMessageBox.showerror("Error", "Python path: '" + pythonPath + "' does not exist.")
			return
		if not os.path.exists(mmaPath):
			tkMessageBox.showerror("Error", "MMA path: '" + mmaPath + "' does not exist.")
		if os.path.isdir(pythonPath):
			tkMessageBox.showerror("Error", "Python path: '" + pythonPath + "' is a directory and not a filename.")
			return
		if os.path.isdir(mmaPath):
			tkMessageBox.showerror("Error", "MMA path: '" + mmaPath + "' is a directory and not a filename.")
			return

		if not os.path.exists(libDir):
			tkMessageBox.showerror("Error", "MMA grooves path: '" + libDir + "' does not exist.")
			return

		realmidipath = re.compile(r'\s-[^\s]*').sub("", midiPlayer)	# strip any command-line options
		if not os.path.exists(realmidipath):
			tkMessageBox.showwarning("Warning", "Midi player path: '" + midiPlayer + "' does not exist. You won't be able to play MMA files.")
		if os.path.isdir(realmidipath):
			tkMessageBox.showwarning("Warning", "Midi player path: '" + midiPlayer + "' is a directory and not a filename. You won't be able to play the MMA file.")

		if customGroovePath != "" and not os.path.exists(customGroovePath):
			tkMessageBox.showwarning("Warning", "Custom groove path: '" + customGroovePath + "' does not exist.")

		# save the settings into settings file
		f = open(settingsFile, "w")
		p = cPickle.Pickler(f)
		p.dump(pythonPath)
		p.dump(mmaPath)
		p.dump(midiPlayer)
		p.dump(customGroovePath)
		p.dump(libDir)
		f.close()

		printOutput("Settings saved in " + settingsFile)


	def refreshGrooves(self):
		global pythonPath
		global mmaPath
		global libDir
		global customGroovePath

		# get the latest paths from GUI
		mmaPath = self.mmaPathEntry.get().strip(' ')
		pythonPath = self.pythonPathEntry.get().strip(' ')
		customGroovePath = self.customGroovePathEntry.get().strip(' ')

		# validate paths
		if not os.path.exists(pythonPath):
			tkMessageBox.showerror("Error", "Python path: '" + pythonPath + "' does not exist. Please check your settings.")
			return
		if not os.path.exists(mmaPath):
			tkMessageBox.showerror("Error", "MMA path: '" + mmaPath + "' does not exist. Please check your settings.")
			return
		if os.path.isdir(pythonPath):
			tkMessageBox.showerror("Error", "Python path: '" + pythonPath + "' is a directory and not a filename. Please check your settings.")
			return
		if os.path.isdir(mmaPath):
			tkMessageBox.showerror("Error", "MMA path: '" + mmaPath + "' is a directory and not a filename. Please check your settings.")
			return

		if customGroovePath != "" and not os.path.exists(customGroovePath):
			tkMessageBox.showwarning("Warning", "Custom groove path: '" + customGroovePath + "' does not exist. These will not be added.")

		mmaDir = os.path.dirname(mmaPath)
		#mmaDir = re.compile(r'.*[\\/]').match(mmaPath).group()
		if libDir == "" or not os.path.exists(libDir):
			printOutput("Couldn't find the MMA grooves directory. Please check your settings.")
			return

		# clear out the grooves
		global groovedict
		global groovelib_lookup
		global groovesFile

		groovedict = {}
		groovelib_lookup = {}

		self.addGrooves(libDir)
		if customGroovePath != "":
			self.addGrooves(customGroovePath)
		#print groovedict
		#write groovedict to grooves file
		f = open(groovesFile, "w")
		p = cPickle.Pickler(f)
		p.dump(groovedict)
		p.dump(groovelib_lookup)
		f.close()


	def addGrooves(self, currlibDir):
		global mmaPath
		global libDir
		global pythonPath
		# get all groove files starting from currlibDir, note this takes a while to complete
		mmaDir = os.path.dirname(mmaPath)
		#mmaDir = re.compile(r'.*[\\/]').match(mmaPath).group()

		tree = os.walk(currlibDir)
		currentdir = os.getcwd()
		os.chdir(mmaDir)
		
		global groovedict
		global outputWindow
					
		groovecount = 0
		p1 = re.compile(r'\\filehead\{(.*?)\}\{(.*?)\}')
		p2 = re.compile(r'\\instable\{(.*?)\}\{(.*?)\}')
		
		# check MMA version
		mmaversion = 1.2
		versionstr = ""
		try:
			cmd = pythonPath + " " + mmaPath + " -v"
			#pipe = os.popen(cmd, 'r')
			pipe = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE).stdout
			output = pipe.read()
			versionstr = re.compile(r'^\d+\.\d+', re.S|re.M).search(output).group(0)
			mmaversion = float(versionstr)
			status = pipe.close()
			printOutput("Detected MMA version " + versionstr)
			logging.debug("[addGrooves] Detected MMA version " + versionstr)
		except:
			printOutput("Couldn't get MMA version! '" + cmd + "' Assuming version > 1.0")
			logging.debug("[addGrooves] Unable to get MMA version. Using '" + cmd + "'")

		# set the path part of the key to groovedict and groovelib_lookup
		# we'll define it as "<default>" if currlibDir matches the current global libDir
		# with this definition, currently loaded files will be less affected if libDir changes (when refreshing grooves)
		if currlibDir == libDir:
			keylibDir = "<default>"
		else:
			keylibDir = currlibDir

		# start walking the directory
		for directory in tree:
			for file in directory[2]:
				if file.endswith("mma"):
					filename = directory[0] + os.sep + file
					# run MMA command
					try:
						# NOTE: In Windows, I need a shell window for this to work!
						if mmaversion > 1.0:
							cmd = pythonPath + " " + mmaPath + " -Dxl -w " + filename
						else:
							cmd = pythonPath + " " + mmaPath + " -Dx -w " + filename
						pipe = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE).stdout
						logging.debug("[addGrooves] Calling subprocess '" + cmd + "'")
						#pipe = os.popen(cmd, 'r')
						grooveinfo = pipe.read()
						status = pipe.close()
						
						#print grooveinfo
					except:
						printOutput("Oops, an error was encountered")
						logging.debug("[addGrooves] Error encountered trying to run '" + cmd + "'")
						
					m1 = p1.search(grooveinfo)
					groovelibname = m1.group(1)
					groovelibrary = groovelibname + "|" + keylibDir

					groovedict[groovelibrary] = [m1.group(2), directory[0] + os.sep + file]
					
					#print groovedict
					m2 = p2.findall(grooveinfo)
					grooves = []
					
					printOutput("[MMA " + versionstr + "] Reading grooves from " + currlibDir + "...\n\n")
					
					for groove in m2:	
						groovename = groove[0]
						groovedesc = groove[1]
						
						groovename = groovename.replace("\\\\","\\")
						groovename = groovename.replace("\\&","&")
						groovename = groovename.replace("``","`")
						groovename = groovename.replace("\"\"","\"")
						groovename = groovename.replace("\'\'","\'")
						
						groovedesc = groovedesc.replace("\\\\","\\")
						groovedesc = groovedesc.replace("\\&","&")
						groovedesc = groovedesc.replace("``","`")
						groovedesc = groovedesc.replace("\"\"","\"")
						groovedesc = groovedesc.replace("\'\'","\'")
						
						#print groovename
						
						outputWindow.insert("2.end", " "+groovename)
						groovecount += 1
						
						grooves += [[groovename, groovedesc]]
						groovelib_lookup[groovename+"|"+keylibDir] = groovelibname
					groovedict[groovelibrary] += [grooves]
					
					outputWindow.update()
		os.chdir(currentdir)
		outputWindow.delete("2.0", "2.end")
		outputWindow.insert("1.end", " (read "+str(groovecount)+" grooves)")
		outputWindow.update()
		#print groovelib_lookup
		

class GroovesDialog(tkSimpleDialog.Dialog):
	selectedGroove = ""
	selectedGroove_lib = ""
	selectedGroove_libpath = ""
	selectedGroove_autolibpath = ""
	currentgroove = ""

	def __init__(self, parent, groove=""):
		self.currentgroove = groove
		tkSimpleDialog.Dialog.__init__(self, parent, title="Select Groove")


	def body(self, master):
		global groovedict
		global groovelib_lookup
		global currentgroovelib
		global libDir

		# If there are no grooves in the dictionary, display error message and return
		if groovedict == {}:
			self.label = Label(master, text="No grooves yet! Please check MMA path setting and refresh grooves.")
			self.label.grid(row=0, column=0, sticky=N+S+E+W)
			return

		logging.debug("[GroovesDialog] Current groove in measure is: " + self.currentgroove + " at " + currentgroovelib)
		
		# If there are grooves, proceed
		keylist = groovedict.keys()

		keylist.sort()
		if currentgroovelib == "":
			currentgroovelib = keylist[0]
		else:
			if currentgroovelib not in groovedict.keys():
				logging.debug("[GroovesDialog] Current groove " + self.currentgroove + " at " + currentgroovelib + " is not found.")
				self.label = Label(master, text="Groove not found in grooves dictionary")
				self.label.grid(row=0, column=0, sticky=N+S+E+W)
				return

		[groovelibname, groovelibpath] = currentgroovelib.split("|")

		self.mb = Menubutton(master, text=groovelibname, direction=RIGHT, relief=RAISED)
		self.mb.grid(row=0, column=0, sticky=NW)
	
		# the groove library names go into a menu
		self.mb.menu = Menu(self.mb, tearoff=0)
		self.mb["menu"] = self.mb.menu

		# define several cascade menus, we'll sort by alphabet
		menus = {}
		menuitems = {}
		for ch in "#ABCDEFGHIJKLMNOPQRSTUVWXYZ":
			menus[ch] = Menu(self.mb, tearoff=0)
			menuitems[ch] = 0
			self.mb.menu.add_cascade(menu=menus[ch], label=ch)
		
		for key in keylist:
			[groovelibname, groovelibpath] = key.split("|")
			#print groovelibname, groovelibpath

			def updateListBox(key=key):
				self.__updateListBox(key)

			if key[0] in "0123456789":
				menus["#"].add_command(label=groovelibname, command=updateListBox)
				menuitems["#"] += 1
			elif key[0].upper() in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
				menus[key[0].upper()].add_command(label=groovelibname, command=updateListBox)
				menuitems[key[0].upper()] += 1
			else:
				logging.debug("[GroovesDialog] " + groovelibname + " does not begin with alphanumeric character")
			#self.mb.menu.add_command(label=key, command=updateListBox)
		# disable the cascade menus which are empty
		index = 0
		for ch in "#ABCDEFGHIJKLMNOPQRSTUVWXYZ":
			if menuitems[ch] == 0:
				self.mb.menu.entryconfigure(index, state=DISABLED)
			index += 1

		# which refreshes a listbox (with description)
		self.label = Label(master, text="Description: "+groovedict[currentgroovelib][0], anchor=NW, justify=LEFT, pady=10, wraplength=100)
		self.label.grid(row=1, column=0, sticky=N+S+E+W)
		
		self.lb = Listbox(master, width=50, height=15)
		location = groovedict[currentgroovelib][1]
		grooves = groovedict[currentgroovelib][2]
		grooves.sort()
		i = 0
		for groove in grooves:
			self.lb.insert(END, groove[0] + ": " + groove[1])
			if self.currentgroove == groove[0]:
				self.lb.selection_set(END)
			i = i + 1

		self.lb.grid(row=2, column=0, stick=N+S+E+W)

		self.yScroll = AutoScrollbar(master, orient=VERTICAL)
		self.yScroll.grid(row=2, column=1, sticky=N+S)

		self.xScroll = AutoScrollbar(master, orient=HORIZONTAL)
		self.xScroll.grid(row=3, column=0, sticky=E+W)

		self.lb["xscrollcommand"] = self.xScroll.set
		self.lb["yscrollcommand"] = self.yScroll.set
		self.xScroll["command"] = self.lb.xview
		self.yScroll["command"] = self.lb.yview

		[groovelibname, libPath] = currentgroovelib.split("|")

		if libPath == "<default>":
			libPath = libDir
		libFile = location.replace(libPath + os.sep, "")

		self.libPathLabel = Label(master, text="Library Path: " + libPath, anchor=NW, justify=LEFT, pady=0, wraplength=100)
		self.libPathLabel.grid(row=4, column=0, sticky=N+S+E+W)

		self.libFileLabel = Label(master, text="Library File: " + libFile, anchor=NW, justify=LEFT, pady=0, wraplength=100)
		self.libFileLabel.grid(row=5, column=0, sticky=N+S+E+W)

		self.update_idletasks()
		self.label.configure(wraplength=self.lb.winfo_width())
		self.libPathLabel.configure(wraplength=self.lb.winfo_width())
		self.libFileLabel.configure(wraplength=self.lb.winfo_width())
		self.lb.configure(height=min(self.lb.size(), 15))
		
	def __updateListBox(self, key):
		global groovedict
		global currentgroovelib
		if key != currentgroovelib:
			self.lb.delete(0, END)
			location = groovedict[key][1]
			grooves = groovedict[key][2]
			grooves.sort()
			for groove in grooves:
				self.lb.insert(END, groove[0] + ": " + groove[1])
			currentgroovelib = key
			[groovelibname, groovelibpath] = currentgroovelib.split("|")
			self.mb.configure(text=groovelibname)

			self.label.configure(text="Description: "+groovedict[key][0])

			[groovelibname, libPath] = currentgroovelib.split("|")
			if libPath == "<default>":
				libPath = libDir

			libFile = location.replace(libPath + os.sep, "")
			self.libPathLabel.configure(text="Library Path: " + libPath)
			self.libFileLabel.configure(text="Library File: " + libFile)
			
			self.update_idletasks()
			#self.lb.configure(width=50)
			self.lb.configure(height=min(self.lb.size(), 15))

	def apply(self):
		global groovedict
		global libDir

		# If no grooves, return nothing
		if groovedict == {}:
			selected = ""
			return

		# If ok, get listbox selection
		if self.lb.curselection() != ():
			# groove name is taken from listbox
			selected = self.lb.get(self.lb.curselection())
			self.selectedGroove = re.compile(r':.*').sub("", selected)

			# groove lib name
			[self.selectedGroove_lib, groovelibpath] = currentgroovelib.split("|")

			# determine libpath setting
			temp = self.libPathLabel["text"]
			temp = temp.replace("Library Path: ", "")
			self.selectedGroove_libpath = temp

			# determine the autolibpath setting
			temp = self.libFileLabel["text"]
			temp = temp.replace("Library File: ", "")
			temp = self.selectedGroove_libpath + os.sep + temp
			temp = os.path.dirname(temp)
			temp = temp.replace(libDir + os.sep, "")
			if temp == self.selectedGroove_libpath:
				self.selectedGroove_autolibpath = "."
			else:
				self.selectedGroove_autolibpath = temp

			# set selected groove libpath to "<default>" if it matches libDir
			if self.selectedGroove_libpath == libDir:
				self.selectedGroove_libpath = "<default>"

