#!/usr/bin/env python

# Script to install lemma

import os
import sys
import platform
import shutil
import stat
import getopt

PREFIX = "/usr/local"
NOPROMPT = False
errors = 0

def prompt():
	input=raw_input("...Press [ENTER] to continue, anything else to terminate: ")
	if input:
		print "Terminated by user..."
		sys.exit(1)
	else:
		print

# Check for command-line arguments

try:
	opts, args = getopt.getopt(sys.argv[1:], "h", ["prefix=", "help", "noprompt"])
except getopt.GetoptError, err:
	# print help information and exit:
	print str(err)
	print

for o, a in opts:
	if o in ("--prefix"):
		if a != "":
			PREFIX = a.rstrip('/')
	if o in ("--noprompt"):
		NOPROMPT = True
	if o in ("--help", "-h"):
		print "Usage: install.py [--prefix=... (default: /usr/local)] [--noprompt]"
		sys.exit()

# Set the paths
BINDIR = PREFIX + "/bin"
LIBDIR = PREFIX + "/share/lemma"

print "This script will install LeMMA in " + BINDIR + " and " + LIBDIR

# Check Linux platform
if platform.system() in ("Windows", "Microsoft"):
	print "You are running Windows. This script is for Linux."
	print "To install in Windows, just copy the files to a directory of your choice"
	sys.exit(1)

# Warn if not running as root user
user = os.getuid()

if user != 0:
	print "[Warning] You should run this script as root user. Continue anyway?"

if not NOPROMPT:
	prompt()

# Check if BINDIR exists (shouldn't need this, just in case...)
if not (os.path.exists(BINDIR) and os.path.isdir(BINDIR)):
	print "[Warning] Directory " + BINDIR + " does not exist. Should I proceed?"
	if not NOPROMPT:
		prompt()
	# Create the BINDIR
	try:
		os.makedirs(BINDIR)
	except:
		# No point continuing now...
		print "[Error] Unable to create " + BINDIR
		sys.exit(1)

# Copy LeMMA binary
print "Copying lemma.py to " + BINDIR + "/lemma"
try:
	shutil.copy("lemma.py", BINDIR + "/lemma")
	os.chmod(BINDIR + "/lemma", 0755)
except:
	print "[Error] Unable to copy!"
	errors += 1

# Check if LIBDIR exists
if os.path.exists(LIBDIR) and os.path.isdir(LIBDIR):
	print "[Warning] Directory " + LIBDIR + " exist. Should I proceed?"
	if not NOPROMPT:
		prompt()
else:
	# Create the LIBDIR
	try:
		os.makedirs(LIBDIR)
	except:
		print "[Warning] Unable to create " + LIBDIR
		sys.exit(1)

# In case LIBDIR/LeMMA is missing
try:
	os.makedirs(LIBDIR + "/LeMMA")
except:
	print

# LeMMA files and modules
filelist = ["README", "CHANGES", "help.txt", "LeMMA/__init__.py", "LeMMA/app.py", "LeMMA/settings.py", "LeMMA/constants.py"]
for filename in filelist:
	try:
		print "Copying " + filename + " to " + LIBDIR + "/" + filename
		shutil.copy(filename, LIBDIR + "/" + filename)
	except:
		print "[Error] Unable to copy!"
		errors += 1

if errors > 0:
	print "\nErrors encountered during installation."
else:
	print "\nInstallation complete."

print "\nIf you want to uninstall LeMMA, just delete the file " + BINDIR + "/lemma and the directory " + LIBDIR + "\n"

print "Type \"lemma\" to run."

