#!/usr/bin/env python

"""
LeMMA - the Lazy man's Editor for MMA
Note that this is NOT really an MMA editor, but rather a simple front-end to MMA

Please read help.txt and CHANGES for more information.

Version 0.7.1. Copyright (c) Gek S. Low. 26 Mar 2009

Send your comments & suggestions to geksiong@yahoo.com
"""

import getopt
import logging
import os
import sys

modulepath = ""

# Where to look for LeMMA modules
for d in ("/usr/local/share/lemma", "/usr/share/lemma", os.path.abspath(os.path.dirname(__file__))):
	if os.path.isdir(d):
		sys.path.insert(0, d)
		modulepath = d
		break

from LeMMA.app import *
from LeMMA.settings import *

def printHelp():
	print title
	print version
	print copyright
	print """
Usage: lemma(.py) [options] [MMA file]

Command-line options:
	--help		This help message
	--debug		Turns on debug mode (run this from a shell window)
	--config=dir	Use "dir" for configuration settings
	"""

def main():
	loglevel = logging.ERROR
	configpath = ""

	# Look for any command-line options
	try:
		opts, args = getopt.getopt(sys.argv[1:], "h", ["debug", "config=", "help"])
	except getopt.GetoptError, err:
		# print help information and exit:
		print str(err)
		print
	for o, a in opts:
		if o in ("--debug"):
			loglevel = logging.DEBUG
		if o in ("--config"):
			configpath = a
		if o in ("--help", "-h"):
			printHelp()
			sys.exit()

	logging.basicConfig(level=loglevel, format="%(levelname)s:%(module)s:%(lineno)d:%(message)s")
	logging.debug("[main] LeMMA " + version + " - debug mode activated")
	logging.debug("[main] Module path found at " + modulepath)
	if configpath != "":
		logging.debug("[main] User-defined config path: " + configpath)

	# any file to load?
	filename = ""
	if args != []:
		filename = os.path.normpath(args[0])
		if os.path.exists(filename) and not os.path.isdir(filename):
			logging.debug("Filename provided at command-line: " + filename)
		else:
			logging.debug("Invalid filename provided at command-line: " + filename)
			filename = ""

	# Main
	app = Application(lemma_configpath=configpath, lemma_modulepath=modulepath)
	if filename != "":
		app.currentfile = filename
		app.loadMMA(filename)
	app.mainloop()

if __name__ == "__main__":
	main()
